@@ -20,16 +20,15 @@
 #include <sys/xattr.h>
 #include <stdlib.h>
 #include <string.h>
-#include <stdio.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <dirent.h>
 #include <errno.h>
-#include <getopt.h>
 #include <stdbool.h>
 #include <uuid/uuid.h>
 #include "libbtrfsutil/btrfsutil.h"
-#include "kernel-shared/ctree.h"
+#include "kernel-shared/uapi/btrfs.h"
+#include "kernel-shared/uapi/btrfs_tree.h"
 #include "common/defs.h"
 #include "common/messages.h"
 #include "common/open-utils.h"
@@ -38,7 +37,6 @@
 #include "common/filesystem-utils.h"
 #include "cmds/commands.h"
 #include "cmds/props.h"
-#include "ioctl.h"
 
 #define XATTR_BTRFS_PREFIX     "btrfs."
 #define XATTR_BTRFS_PREFIX_LEN (sizeof(XATTR_BTRFS_PREFIX) - 1)
@@ -288,6 +286,19 @@ static int check_btrfs_object(const char *object)
 {
 	int ret;
 	u8 fsid[BTRFS_FSID_SIZE];
+	struct stat st;
+
+	ret = stat(object, &st);
+	if (ret < 0)
+		return 0;
+
+	/*
+	 * Don't try to read fsid on char devices, this will fail anyway. In
+	 * some cases opening the device can trigger some system events (like a
+	 * watchdog) so do only stat.
+	 */
+	if ((st.st_mode & S_IFMT) == S_IFCHR || (st.st_mode & S_IFMT) == S_IFIFO)
+		return 0;
 
 	ret = get_fsid(object, fsid, 0);
 	if (ret < 0)
