@@ -25,6 +25,7 @@
 #include <strings.h>
 #include "libbtrfsutil/btrfsutil.h"
 #include "kernel-shared/volumes.h"
+#include "kernel-shared/ctree.h"
 #include "kernel-shared/compression.h"
 #include "common/parse-utils.h"
 #include "common/messages.h"
@@ -84,9 +85,7 @@ int parse_range(const char *range, u64 *start, u64 *end)
 	}
 
 	if (*start > *end) {
-		error("range %llu..%llu doesn't make sense",
-			(unsigned long long)*start,
-			(unsigned long long)*end);
+		error("range %llu..%llu doesn't make sense", *start, *end);
 		return 1;
 	}
 
@@ -135,9 +134,7 @@ int parse_range_strict(const char *range, u64 *start, u64 *end)
 {
 	if (parse_range(range, start, end) == 0) {
 		if (*start >= *end) {
-			error("range %llu..%llu not allowed",
-				(unsigned long long)*start,
-				(unsigned long long)*end);
+			error("range %llu..%llu not allowed", *start, *end);
 			return 1;
 		}
 		return 0;
@@ -161,6 +158,7 @@ u64 parse_size_from_string(const char *s)
 		error("size value '%s' is less equal than 0", s);
 		exit(1);
 	}
+	errno = 0;
 	ret = strtoull(s, &endptr, 10);
 	if (endptr == s) {
 		error("size value '%s' is invalid", s);
@@ -184,22 +182,22 @@ u64 parse_size_from_string(const char *s)
 		switch (c) {
 		case 'e':
 			mult *= 1024;
-			/* fallthrough */
+			fallthrough;
 		case 'p':
 			mult *= 1024;
-			/* fallthrough */
+			fallthrough;
 		case 't':
 			mult *= 1024;
-			/* fallthrough */
+			fallthrough;
 		case 'g':
 			mult *= 1024;
-			/* fallthrough */
+			fallthrough;
 		case 'm':
 			mult *= 1024;
-			/* fallthrough */
+			fallthrough;
 		case 'k':
 			mult *= 1024;
-			/* fallthrough */
+			fallthrough;
 		case 'b':
 			break;
 		default:
