@@ -50,10 +50,14 @@ int path_is_block_device(const char *path)
 }
 
 /*
- * check if given path is a mount point
- * return 1 if yes. 0 if no. -1 for error
+ * Check if given path is a mount point. (Note: a similar function also exists
+ * in libudev so it's been renamed to avoid clash.)
+ *
+ * Return: 1 if yes,
+ *         0 if no,
+ *        -1 for error
  */
-int path_is_mount_point(const char *path)
+int path_is_a_mount_point(const char *path)
 {
 	FILE *f;
 	struct mntent *mnt;
