@@ -18,6 +18,7 @@
 #define CRYPTO_HASH_H
 
 #include "kerncompat.h"
+#include <stddef.h>
 
 #define CRYPTO_HASH_SIZE_MAX	32
 
@@ -28,7 +29,5 @@ int hash_blake2b(const u8 *buf, size_t length, u8 *out);
 
 void hash_init_accel(void);
 void hash_init_crc32c(void);
-void hash_init_blake2(void);
-void hash_init_sha256(void);
 
 #endif
